# Copyright 2009 Canonical Ltd.  All rights reserved.

"""Martian directives used in lazr.restful."""

__metaclass__ = type
__all__ = ['request_class',
           'publication_class',
           'settings',]

import martian
from zope.component import getSiteManager, getUtility
from zope.location.interfaces import ILocation
from zope.traversing.browser import AbsoluteURL
from zope.traversing.browser.interfaces import IAbsoluteURL

from lazr.restful.interfaces import (
    IServiceRootResource, IWebServiceConfiguration, IWebServiceLayer)
from lazr.restful import ServiceRootResource
from lazr.restful.simple import (
    BaseWebServiceConfiguration, Publication, Request,
    RootResourceAbsoluteURL)


class request_class(martian.Directive):
    """Directive used to specify a service's request class.

    The default is lazr.restful.simple.Request.
    """
    scope = martian.CLASS
    store = martian.ONCE
    default = Request


class publication_class(martian.Directive):
    """Directive used to specify a service's publication class.

    The default is lazr.restful.simple.Publication.
    """
    scope = martian.CLASS
    store = martian.ONCE
    default = Publication


class ConfigurationGrokker(martian.ClassGrokker):
    """Subclass BaseWebServiceConfiguration for an easy configuration class.

    Subclass BaseWebServiceConfiguration and override any of the
    configuration settings specific to your web service.

    This grokker creates default implementations of createRequest()
    and get_request_user, if those methods are not implemented.

    The default implementations use the request and publication
    classes you specify with the directives
    lazr.restful.directives.request_class() and
    lazr.restful.directives.publication_class().

    This grokker then registers an instance of your subclass as the
    singleton configuration object.
    """
    martian.component(BaseWebServiceConfiguration)
    martian.directive(request_class)
    martian.directive(publication_class)
    def execute(self, cls, config, request_class,
                publication_class, *kw):
        # If implementations of the IWebServiceConfiguration methods are
        # missing, create them from the declarations.
        if not getattr(cls, 'createRequest', None):
            def createRequest(self, body_instream, environ):
                """See `IWebServiceConfiguration`."""
                request = request_class(body_instream, environ)
                service_root_object = getUtility(IServiceRootResource)
                request.setPublication(publication_class(service_root_object))
                return request
            cls.createRequest = createRequest

        if not getattr(cls, 'get_request_user', None):
            def get_request_user(self):
                return None
            cls.get_request_user = get_request_user

        # Register as utility.
        getSiteManager().registerUtility(cls(), IWebServiceConfiguration)
        return True


class ServiceRootGrokker(martian.ClassGrokker):
    """Registers your service root as a singleton object."""
    martian.component(ServiceRootResource)
    def execute(self, cls, config, *kw):
        getSiteManager().registerUtility(cls(), IServiceRootResource)
        return True


class RootResourceAbsoluteURLGrokker(martian.ClassGrokker):
    """Registers a strategy for generating your service's base URL.

    In most cases you can simply create an empty subclass of
    RootResourceAbsoluteURL.
    """
    martian.component(RootResourceAbsoluteURL)
    def execute(self, cls, config, *kw):
        getSiteManager().registerAdapter(cls)
        return True


class location_interface(martian.Directive):
    """Directive to specify the location interface.

    The default is zope.location.interfaces.ILocation
    """
    scope = martian.CLASS
    store = martian.ONCE
    default = ILocation


class AbsoluteURLGrokker(martian.ClassGrokker):
    """Registers a strategy for generating an object's URL.

    In most cases you can simply create an empty subclass of
    AbsoluteURL.
    """
    martian.component(AbsoluteURL)
    martian.directive(location_interface)
    def execute(self, cls, config, location_interface, *kw):
        getSiteManager().registerAdapter(
            cls, (location_interface, IWebServiceLayer), IAbsoluteURL)
        return True
